/*
 * @(#)SQLGatewayServlet.java  1.0.0 98/01/15
 *
 * Copyright (c) 1998 Sheldon Bradley Wosnick. All Rights Reserved.
 * AUTHOR EMAIL: swosnick@ca.ibm.com
 *
 * DISCLAIMER OF WARRANTIES:
 * The following [enclosed] code is sample code created by Sheldon Bradley
 * Wosnick. This sample code is not part of any standard product and is
 * provided to you solely for the purpose of assisting you in the
 * development of your applications. The code is provided "AS IS", without
 * warranty of any kind. Neither IBM or Sheldon Bradley Wosnick shall be
 * liable for any damages arising out of your use of the sample code, even
 * if advised of the possibility of such damages.
 *
 * The SQLGatewayServlet was entirely developed, debugged and verified with
 * IBM VisualAge for Java Enterprise Edition (Team Beta) and the JavaSoft Java
 * Servlet Development Kit 1.1.
 *
 * CopyrightVersion 1.0.0
 */

package swosnick.database.servlets;

import java.net.URL;
import java.util.*;
import java.sql.*;
import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
 * <p>This is a DEMONSTRATION of a Java servlet which lets you view and manipulate a
 * relational database using SQL via JDBC with any datasource configured on the Web
 * server (including an ODBC datasource). One may use a JDBC driver such as those that
 * ship with DB2 Universal Database 5.0 or the JDBC-ODBC Bridge driver. This servlet
 * supports SQL queries, updates, deletes and inserts. It is not meant to imply that
 * this is the ONLY way, or even the best way, to use servlets with JDBC. It merely
 * attempts to demonstrate one possible way to use Java server-side programs with JDBC
 * that execute on the Web server. It demonstrates the ability to easily connect to
 * a legacy back-end database and move that information swifty over the internet
 * or intranet. With very little modifications, this technology could be adapted to a
 * a variety of applications where remote users send queries, modify and append records
 * over the network to a legacy datasource through any number of intervening tiers or
 * application servers. This is an important aspect of the IBM Network Computing Framework
 * and the e-business strategy. Servlets, then, are an important server-side technology
 * for any form of e-business.
 *
 * <p>The SQLGatewayServlet uses only dynamically created HTML. That is, even the first
 * logon form is generated by the Webserver and additional forms could, of course, be
 * custom-tailored to the particular requests that the server received. This has the
 * potential to give the user a high degree of interactivity, yet preserve the logic
 * on the server.  All HTML forms displayed in the browser as a result of this servlet
 * are generated by the servlet itself. This is possible, in a large part, and unlike
 * traditional CGI programs, because the servlet is able to keep and maintain state
 * information from request to request.
 *
 * <p>Because servlets are written in Java, they can be used on any Java-capable platform.
 * Servlets also provides security reassurances of Java in a networked environment. This
 * way, proprietary algorithms and other intellectual capital are built into a servlet
 * and the code never passes beyond the boundaries of the server -- only the results
 * that it produces does. Since the code is never passed to the client, intellectual
 * capital cannot be simply saved and disassembled.
 *
 * <p>A feature of this servlet is that if the servlet throws an SQLException,
 * information about that exception is HTTP-streamed to the browser for
 * debugging purposes so the user can get meaningful feedback on the SQL-JDBC
 * methods that are being executed improperly. For Java exceptions other than
 * SQLException a Java console is required to accept calls to standard err.
 *
 * <p>All pure JDBC drivers should be supported including the JDBC-ODBC Bridge
 * with the JavaSoft JDK (1.1x). The client HTML form has been constructed to
 * allow for ODBC and DB2, but as new PURE JDBC drivers become available by
 * various database vendors the code can be revised to include those new drivers.
 *
 * <p>PREREQUISITES:<UL>
 *
 * <LI>A web browser capable of displaying tables. Need not support JDK 1.1x.
 *     This is a positive feature - since all logic is operating server-side,
 *     the browser need to be capable of displaying HTML, that's all.
 *
 * <LI>A working to advanced understanding of the SQL language. Knowing how to
 *     build a simple query, update, insert or delete command at a minimum.
 *
 * <LI>Either the ServletRunner which ships with the JavaSoft Java Servlet
 *     Development Kit (JSDK 1.1x) and/or a Web server that supports servlets.
 *     For example, Lotus Domino Go Webserver 4.6x.
 *
 * <LI>An understanding of how to deploy and call a servlet url directly. That is,
 *     "http://<hostname>:<port>/servlet/swosnick.database.servlets.SQLGatewayServet",
 *     where hostname and port are the server and port that host the servlet.
 * </UL>
 *
 * @version  1.0.0 01/15/98
 * @author Sheldon Bradley Wosnick
 * IBM Toronto Lab
 * Toronto Ontario, Canada
 * February 1998
 * email: swosnick@ca.ibm.com
 */

public class SQLGatewayServlet extends HttpServlet {

    /**
     * The user name or ID. Supplied by the user.
     */
    private String userName;

    /**
     * The optional password. Supplied by the user or set to an empty string.
     */
    private String password;

    /**
     * The JDBC-style URL with protocol and datasource name. Supplied by the user.
     */
    private String url;

    /**
     * The JDBC Driver. Supplied by the user from a choice of available drivers.
     */
    private String driver;

    /**
     * The SQL string to use for query or update. Supplied by the user.
     */
       private String sql;

    /**
     * Track the last SQL query or command so that the user need not re-enter it. Used internally only
     */
    private String lastSQL;

    /**
     * The database connection. Used internally only.
     */
    private Connection connection;

    /**
     * Result set for a given query. Used internally only.
     */
    private ResultSet resultSet;

    /**
     * The Statement used for SQL query or update. Used internally only.
     */
    private Statement statement;

    /** Did an exception occur on an SQL action?  Used internally only.
     *
     */
    private boolean isSQLExceptionOccurred = false;

    /** On an executeUpdate operation was it successful - returned rows? Used internally only.
     *
     */
    private boolean isExecuteUpdateSuccess = false;

    /**
     * The output stream used for HTTP output to create dynamic HTML page. Used internally only.
     */
    private ServletOutputStream outstream;

    /**
     * The request sent to the servlet. Used internally only.
     */
    private HttpServletRequest request;

    /**
     * The response prepared by the servlet. Used internally only.
     */
    private HttpServletResponse response;

    /**
     * The title which is sent to the browser for the initial logon form.
     */
    private final String logonFormTitle = "SQL Gateway Servlet Logon Form";

    /**
     * The heading or description for the logon form.
     */
    private final String logonFormHeading = "Welcome to the SQL Gateway Servlet Demonstration";

    /**
     * The name of the servlet used in the logon form HTML to call back the servlet with the servlet tag.
     */

    private final String servletName = "swosnick.database.servlets.SQLGatewayServlet";

    /**
     * The list of supported JDBC drivers which will appear in the logon form as a choice.
     */
    private final String[] JDBCDrivers = {"COM.ibm.db2.jdbc.app.DB2Driver",
                                          "sun.jdbc.odbc.JdbcOdbcDriver"
                                         };
    /**
     * Properties object used to contain username and password to make the database connection.
     * Used internally only.
     */
    private Properties properties;

    /**
     * DEBUG flag - when true, tracing will be enabled, when false, tracing will be disabled and in
     * addition, the compiler is smart enough to remove blocks it knows can never be reached, so the
     * debugging code in effect is never compiled into the executable, with no overhead.
     * Used internally only.
     */
    private boolean isDEBUGON = true; // change as appropriate

    /**
     * Constant representing neither a query or an update, indicating an error retrieving the type of operation.
     * Used internally only.
     */
    private final int NO_OPERATION = 0;

    /**
     * Constant used to represent the SELECT operation which will use executeQuery.
     * Used internally only.
     */
    private final int SELECT = 1;

    /**
     * Constant representing an update, delete or insert operation and will use executeUpdate.
     * Used internally only.
     */
    private final int INSERT_UPDATE_DELETE = 2;

    /**
     * If the user is logged on in appropriately, set to true, else remain false.
     * Used internally only.
     */
    private boolean isLoggedOn = false;

    /**
     * Background color constant for control of background. Change as desired.
     */
    private final String backgroundColor = "#C0C0C0";

    /**
     * Table heading background color constant. Change as desired.
    */
    private final String tableHeadingBackgroundColor = "#993366";

    /**
     * Table heading background color constant. Change as desired.
     */
    private final String tableHeadingForeroundColor ="#FFFFFF";


/**
 * This method was created by a SmartGuide.
 * Checks for and displays SQL warnings. Returns true if a warning existed.
 * @param anSQLwarning SQLWarning
 * @return boolean indicating existence of an SQL warning.
 * @exception SQLException
 */
private static boolean checkForWarning(SQLWarning anSQLwarning) throws SQLException {

    boolean isWarning = false;

    if (anSQLwarning != null) {
        System.err.println("\n *** SQL Warning ***\n");

        isWarning = true;

        while (anSQLwarning != null) {
            System.err.println("SQLState: " +
                anSQLwarning.getSQLState());
            System.err.println("Message:  " +
                anSQLwarning.getMessage());
            System.err.println("Vendor:   " +
                anSQLwarning.getErrorCode());
            System.err.println("");
            anSQLwarning = anSQLwarning.getNextWarning();
        }
    }
    return isWarning;
}
/**
 * This method was created by a SmartGuide.
 * This method will provide information about the driver and the connection useful for database
 * administrators or for trouble shooting. It will publish a lot of the JDBC API methods.
 */
private void connectionInfo() {

    ResultSetMetaData resultSetMetaData = null;
    DatabaseMetaData databaseMetaData = null;

    try {
        resultSetMetaData = resultSet.getMetaData();
        databaseMetaData = connection.getMetaData();

        outstream.println("<font color=#800040>");
        outstream.println("JDBC Driver and Connection Statistics:");
        outstream.println("</font>");
        outstream.println("<BR>");

        outstream.println("<BR>Current date and time: " + new java.util.Date().toString());
        outstream.println("<BR>Connection URL: " + databaseMetaData.getURL());

        outstream.println("<BR>Driver: " +
            databaseMetaData.getDriverName());

        outstream.println("<BR>Version: " +
            databaseMetaData.getDriverVersion());

        outstream.println("<BR>Current User Name: " +
            databaseMetaData.getUserName());

        outstream.println("<BR>Driver Major Version: " +
            databaseMetaData.getDriverMajorVersion());

        outstream.println("<BR>Driver Minor Version: " +
            databaseMetaData.getDriverMinorVersion());

        outstream.println("<BR>Database Product Name: " +
            databaseMetaData.getDatabaseProductName());

        outstream.println("<BR>Database Product Version: " +
            databaseMetaData.getDatabaseProductVersion());

        // Optional statistics for demonstration purposes, comment out if desired.
        outstream.println("<font color=#800040>");
        outstream.println("<BR><BR>Capabilities of the Database:");
        outstream.println("</font>");
        outstream.println("<BR>");

        outstream.println("<BR>allProceduresAreCallable: " + "<font color=#800040>" + databaseMetaData.allProceduresAreCallable() + "</font>");
        outstream.println("<BR>allTablesAreSelectable: " + "<font color=#800040>" + databaseMetaData.allTablesAreSelectable() + "</font>");
        outstream.println("<BR>isReadOnly: " + "<font color=#800040>" + databaseMetaData.isReadOnly() + "</font>");
        outstream.println("<BR>nullsAreSortedHigh: " + "<font color=#800040>" + databaseMetaData.nullsAreSortedHigh() + "</font>");
        outstream.println("<BR>nullsAreSortedLow: " + "<font color=#800040>" + databaseMetaData.nullsAreSortedLow() + "</font>");
        outstream.println("<BR>nullsAreSortedAtStart: " + "<font color=#800040>" + databaseMetaData.nullsAreSortedAtStart() + "</font>");
        outstream.println("<BR>nullsAreSortedAtEnd: " + "<font color=#800040>" + databaseMetaData.nullsAreSortedAtEnd() + "</font>");
        outstream.println("<BR>usesLocalFiles: " + "<font color=#800040>" + databaseMetaData.usesLocalFiles() + "</font>");
        outstream.println("<BR>usesLocalFilePerTable: " + "<font color=#800040>" + databaseMetaData.usesLocalFilePerTable() + "</font>");
        outstream.println("<BR>supportsMixedCaseIdentifiers: " + "<font color=#800040>" + databaseMetaData.supportsMixedCaseIdentifiers() + "</font>");
        outstream.println("<BR>storesUpperCaseIdentifiers: " + "<font color=#800040>" + databaseMetaData.storesUpperCaseIdentifiers() + "</font>");
        outstream.println("<BR>storesLowerCaseIdentifiers: " + "<font color=#800040>" + databaseMetaData.storesLowerCaseIdentifiers() + "</font>");
        outstream.println("<BR>storesMixedCaseIdentifiers: " + "<font color=#800040>" + databaseMetaData.storesMixedCaseIdentifiers() + "</font>");
        outstream.println("<BR>supportsMixedCaseQuotedIdentifiers: " + "<font color=#800040>" + databaseMetaData.supportsMixedCaseQuotedIdentifiers() + "</font>");
        outstream.println("<BR>storesUpperCaseQuotedIdentifiers: " + "<font color=#800040>" + databaseMetaData.storesUpperCaseQuotedIdentifiers() + "</font>");
        outstream.println("<BR>storesLowerCaseQuotedIdentifiers: " + "<font color=#800040>" + databaseMetaData.storesLowerCaseQuotedIdentifiers() + "</font>");
        outstream.println("<BR>storesMixedCaseQuotedIdentifiers: " + "<font color=#800040>" + databaseMetaData.storesMixedCaseQuotedIdentifiers() + "</font>");
        outstream.println("<BR>getIdentifierQuoteString: " + "<font color=#800040>" + databaseMetaData.getIdentifierQuoteString() + "</font>");
        outstream.println("<BR>getSQLKeywords: " + "<font color=#800040>" + databaseMetaData.getSQLKeywords() + "</font>");
        outstream.println("<BR>getNumericFunctions: " + "<font color=#800040>" + databaseMetaData.getNumericFunctions() + "</font>");
        outstream.println("<BR>getStringFunctions: " + "<font color=#800040>" + databaseMetaData.getStringFunctions() + "</font>");
        outstream.println("<BR>getSystemFunctions: " + "<font color=#800040>" + databaseMetaData.getSystemFunctions() + "</font>");
        outstream.println("<BR>getTimeDateFunctions: " + "<font color=#800040>" + databaseMetaData.getTimeDateFunctions() + "</font>");
        outstream.println("<BR>getSearchStringEscape: " + "<font color=#800040>" + databaseMetaData.getSearchStringEscape() + "</font>");
        outstream.println("<BR>getExtraNameCharacters: " + "<font color=#800040>" + databaseMetaData.getExtraNameCharacters() + "</font>");
        outstream.println("<BR>supportsAlterTableWithAddColumn: " + "<font color=#800040>" + databaseMetaData.supportsAlterTableWithAddColumn() + "</font>");
        outstream.println("<BR>supportsAlterTableWithDropColumn: " + "<font color=#800040>" + databaseMetaData.supportsAlterTableWithDropColumn() + "</font>");
        outstream.println("<BR>supportsColumnAliasing: " + "<font color=#800040>" + databaseMetaData.supportsColumnAliasing() + "</font>");
        outstream.println("<BR>nullPlusNonNullIsNull: " + "<font color=#800040>" + databaseMetaData.nullPlusNonNullIsNull() + "</font>");
        outstream.println("<BR>supportsConvert: " + "<font color=#800040>" + databaseMetaData.supportsConvert() + "</font>");
        outstream.println("<BR>supportsTableCorrelationNames: " + "<font color=#800040>" + databaseMetaData.supportsTableCorrelationNames() + "</font>");
        outstream.println("<BR>supportsDifferentTableCorrelationNames: " + "<font color=#800040>" + databaseMetaData.supportsDifferentTableCorrelationNames() + "</font>");
        outstream.println("<BR>supportsExpressionsInOrderBy: " + "<font color=#800040>" + databaseMetaData.supportsExpressionsInOrderBy() + "</font>");
        outstream.println("<BR>supportsOrderByUnrelated: " + "<font color=#800040>" + databaseMetaData.supportsOrderByUnrelated() + "</font>");
        outstream.println("<BR>supportsGroupBy: " + "<font color=#800040>" + databaseMetaData.supportsGroupBy() + "</font>");
        outstream.println("<BR>supportsGroupByUnrelated: " + "<font color=#800040>" + databaseMetaData.supportsGroupByUnrelated() + "</font>");
        outstream.println("<BR>supportsGroupByBeyondSelect: " + "<font color=#800040>" + databaseMetaData.supportsGroupByBeyondSelect() + "</font>");
        outstream.println("<BR>supportsLikeEscapeClause: " + "<font color=#800040>" + databaseMetaData.supportsLikeEscapeClause() + "</font>");
        outstream.println("<BR>supportsMultipleResultSets: " + "<font color=#800040>" + databaseMetaData.supportsMultipleResultSets() + "</font>");
        outstream.println("<BR>supportsMultipleTransactions: " + "<font color=#800040>" + databaseMetaData.supportsMultipleTransactions() + "</font>");
        outstream.println("<BR>supportsNonNullableColumns: " + "<font color=#800040>" + databaseMetaData.supportsNonNullableColumns() + "</font>");
        outstream.println("<BR>supportsMinimumSQLGrammar: " + "<font color=#800040>" + databaseMetaData.supportsMinimumSQLGrammar() + "</font>");
        outstream.println("<BR>supportsCoreSQLGrammar: " + "<font color=#800040>" + databaseMetaData.supportsCoreSQLGrammar() + "</font>");
        outstream.println("<BR>supportsExtendedSQLGrammar: " + "<font color=#800040>" + databaseMetaData.supportsExtendedSQLGrammar() + "</font>");
        outstream.println("<BR>supportsANSI92EntryLevelSQL: " + "<font color=#800040>" + databaseMetaData.supportsANSI92EntryLevelSQL() + "</font>");
        outstream.println("<BR>supportsANSI92IntermediateSQL: " + "<font color=#800040>" + databaseMetaData.supportsANSI92IntermediateSQL() + "</font>");
        outstream.println("<BR>supportsANSI92FullSQL: " + "<font color=#800040>" + databaseMetaData.supportsANSI92FullSQL() + "</font>");
        outstream.println("<BR>supportsIntegrityEnhancementFacility: " + "<font color=#800040>" + databaseMetaData.supportsIntegrityEnhancementFacility() + "</font>");
        outstream.println("<BR>supportsOuterJoins: " + "<font color=#800040>" + databaseMetaData.supportsOuterJoins() + "</font>");
        outstream.println("<BR>supportsFullOuterJoins: " + "<font color=#800040>" + databaseMetaData.supportsFullOuterJoins() + "</font>");
        outstream.println("<BR>supportsLimitedOuterJoins: " + "<font color=#800040>" + databaseMetaData.supportsLimitedOuterJoins() + "</font>");
        outstream.println("<BR>getSchemaTerm: " + "<font color=#800040>" + databaseMetaData.getSchemaTerm() + "</font>");
        outstream.println("<BR>getProcedureTerm: " + "<font color=#800040>" + databaseMetaData.getProcedureTerm() + "</font>");
        outstream.println("<BR>getCatalogTerm: " + "<font color=#800040>" + databaseMetaData.getCatalogTerm() + "</font>");
        outstream.println("<BR>isCatalogAtStart: " + "<font color=#800040>" + databaseMetaData.isCatalogAtStart() + "</font>");
        outstream.println("<BR>getCatalogSeparator: " + "<font color=#800040>" + databaseMetaData.getCatalogSeparator() + "</font>");
        outstream.println("<BR>supportsSchemasInDataManipulation: " + "<font color=#800040>" + databaseMetaData.supportsSchemasInDataManipulation() + "</font>");
        outstream.println("<BR>supportsSchemasInProcedureCalls: " + "<font color=#800040>" + databaseMetaData.supportsSchemasInProcedureCalls() + "</font>");
        outstream.println("<BR>supportsSchemasInTableDefinitions: " + "<font color=#800040>" + databaseMetaData.supportsSchemasInTableDefinitions() + "</font>");
        outstream.println("<BR>supportsSchemasInIndexDefinitions: " + "<font color=#800040>" + databaseMetaData.supportsSchemasInIndexDefinitions() + "</font>");
        outstream.println("<BR>supportsSchemasInPrivilegeDefinitions: " + "<font color=#800040>" + databaseMetaData.supportsSchemasInPrivilegeDefinitions() + "</font>");
        outstream.println("<BR>supportsCatalogsInDataManipulation: " + "<font color=#800040>" + databaseMetaData.supportsCatalogsInDataManipulation() + "</font>");
        outstream.println("<BR>supportsCatalogsInProcedureCalls: " + "<font color=#800040>" + databaseMetaData.supportsCatalogsInProcedureCalls() + "</font>");
        outstream.println("<BR>supportsCatalogsInTableDefinitions: " + "<font color=#800040>" + databaseMetaData.supportsCatalogsInTableDefinitions() + "</font>");
        outstream.println("<BR>supportsCatalogsInIndexDefinitions: " + "<font color=#800040>" + databaseMetaData.supportsCatalogsInIndexDefinitions() + "</font>");
        outstream.println("<BR>supportsCatalogsInPrivilegeDefinitions: " + "<font color=#800040>" + databaseMetaData.supportsCatalogsInPrivilegeDefinitions() + "</font>");
        outstream.println("<BR>supportsPositionedDelete: " + "<font color=#800040>" + databaseMetaData.supportsPositionedDelete() + "</font>");
        outstream.println("<BR>supportsPositionedUpdate: " + "<font color=#800040>" + databaseMetaData.supportsPositionedUpdate() + "</font>");
        outstream.println("<BR>supportsSelectForUpdate: " + "<font color=#800040>" + databaseMetaData.supportsSelectForUpdate() + "</font>");
        outstream.println("<BR>supportsStoredProcedures: " + "<font color=#800040>" + databaseMetaData.supportsStoredProcedures() + "</font>");
        outstream.println("<BR>supportsSubqueriesInComparisons: " + "<font color=#800040>" + databaseMetaData.supportsSubqueriesInComparisons() + "</font>");
        outstream.println("<BR>supportsSubqueriesInExists: " + "<font color=#800040>" + databaseMetaData.supportsSubqueriesInExists() + "</font>");
        outstream.println("<BR>supportsSubqueriesInIns: " + "<font color=#800040>" + databaseMetaData.supportsSubqueriesInIns() + "</font>");
        outstream.println("<BR>supportsSubqueriesInQuantifieds: " + "<font color=#800040>" + databaseMetaData.supportsSubqueriesInQuantifieds() + "</font>");
        outstream.println("<BR>supportsCorrelatedSubqueries: " + "<font color=#800040>" + databaseMetaData.supportsCorrelatedSubqueries() + "</font>");
        outstream.println("<BR>supportsUnion: " + "<font color=#800040>" + databaseMetaData.supportsUnion() + "</font>");
        outstream.println("<BR>supportsUnionAll: " + "<font color=#800040>" + databaseMetaData.supportsUnionAll() + "</font>");
        outstream.println("<BR>supportsOpenCursorsAcrossCommit: " + "<font color=#800040>" + databaseMetaData.supportsOpenCursorsAcrossCommit() + "</font>");
        outstream.println("<BR>supportsOpenCursorsAcrossRollback: " + "<font color=#800040>" + databaseMetaData.supportsOpenCursorsAcrossRollback() + "</font>");
        outstream.println("<BR>supportsOpenStatementsAcrossCommit: " + "<font color=#800040>" + databaseMetaData.supportsOpenStatementsAcrossCommit() + "</font>");
        outstream.println("<BR>supportsOpenStatementsAcrossRollback: " + "<font color=#800040>" + databaseMetaData.supportsOpenStatementsAcrossRollback() + "</font>");
        outstream.println("<BR>getMaxBinaryLiteralLength: " + "<font color=#800040>" + databaseMetaData.getMaxBinaryLiteralLength() + "</font>");
        outstream.println("<BR>getMaxCharLiteralLength: " + "<font color=#800040>" + databaseMetaData.getMaxCharLiteralLength() + "</font>");
        outstream.println("<BR>getMaxColumnNameLength: " + "<font color=#800040>" + databaseMetaData.getMaxColumnNameLength() + "</font>");
        outstream.println("<BR>getMaxColumnsInGroupBy: " + "<font color=#800040>" + databaseMetaData.getMaxColumnsInGroupBy() + "</font>");
        outstream.println("<BR>getMaxColumnsInIndex: " + "<font color=#800040>" + databaseMetaData.getMaxColumnsInIndex() + "</font>");
        outstream.println("<BR>getMaxColumnsInOrderBy: " + "<font color=#800040>" + databaseMetaData.getMaxColumnsInOrderBy() + "</font>");
        outstream.println("<BR>getMaxColumnsInSelect: " + "<font color=#800040>" + databaseMetaData.getMaxColumnsInSelect() + "</font>");
        outstream.println("<BR>getMaxColumnsInTable: " + "<font color=#800040>" + databaseMetaData.getMaxColumnsInTable() + "</font>");
        outstream.println("<BR>getMaxConnections: " + "<font color=#800040>" + databaseMetaData.getMaxConnections() + "</font>");
        outstream.println("<BR>getMaxCursorNameLength: " + "<font color=#800040>" + databaseMetaData.getMaxCursorNameLength() + "</font>");
        outstream.println("<BR>getMaxIndexLength: " + "<font color=#800040>" + databaseMetaData.getMaxIndexLength() + "</font>");
        outstream.println("<BR>getMaxSchemaNameLength: " + "<font color=#800040>" + databaseMetaData.getMaxSchemaNameLength() + "</font>");
        outstream.println("<BR>getMaxProcedureNameLength: " + "<font color=#800040>" + databaseMetaData.getMaxProcedureNameLength() + "</font>");
        outstream.println("<BR>getMaxCatalogNameLength: " + "<font color=#800040>" + databaseMetaData.getMaxCatalogNameLength() + "</font>");
        outstream.println("<BR>getMaxRowSize: " + "<font color=#800040>" + databaseMetaData.getMaxRowSize() + "</font>");
        outstream.println("<BR>doesMaxRowSizeIncludeBlobs: " + "<font color=#800040>" + databaseMetaData.doesMaxRowSizeIncludeBlobs() + "</font>");
        outstream.println("<BR>getMaxStatementLength: " + "<font color=#800040>" + databaseMetaData.getMaxStatementLength() + "</font>");
        outstream.println("<BR>getMaxStatements: " + "<font color=#800040>" + databaseMetaData.getMaxStatements() + "</font>");
        outstream.println("<BR>getMaxTableNameLength: " + "<font color=#800040>" + databaseMetaData.getMaxTableNameLength() + "</font>");
        outstream.println("<BR>getMaxTablesInSelect: " + "<font color=#800040>" + databaseMetaData.getMaxTablesInSelect() + "</font>");
        outstream.println("<BR>getMaxUserNameLength: " + "<font color=#800040>" + databaseMetaData.getMaxUserNameLength() + "</font>");
        outstream.println("<BR>getDefaultTransactionIsolation: " + "<font color=#800040>" + databaseMetaData.getDefaultTransactionIsolation() + "</font>");
        outstream.println("<BR>supportsTransactions: " + "<font color=#800040>" + databaseMetaData.supportsTransactions() + "</font>");
        outstream.println("<BR>supportsDataDefinitionAndDataManipulationTransactions: " + "<font color=#800040>" + databaseMetaData.supportsDataDefinitionAndDataManipulationTransactions() + "</font>");
        outstream.println("<BR>supportsDataManipulationTransactionsOnly: " + "<font color=#800040>" + databaseMetaData.supportsDataManipulationTransactionsOnly() + "</font>");
        outstream.println("<BR>dataDefinitionCausesTransactionCommit: " + "<font color=#800040>" + databaseMetaData.dataDefinitionCausesTransactionCommit() + "</font>");
        outstream.println("<BR>dataDefinitionIgnoredInTransactions: " + "<font color=#800040>" + databaseMetaData.dataDefinitionIgnoredInTransactions() + "</font>");
        outstream.println("<BR>");
        outstream.println("<HR WIDTH=100%>");
    }
    catch (SQLException anSQLException) {
        if (isDEBUGON) trace("An SQL Exception was thrown:\n" + anSQLException);
            processSQLException(anSQLException);
    }
    catch (IOException anIOException) {
        System.err.println ("\n*** IOException caught ***\n");
        System.err.println("IOException: " + anIOException.toString());
    }
}
/**
 * This method was created by a SmartGuide.
 * Perform the physical connection to the database.
 */
private synchronized void connectToDatabase() {

    // Note: by default, JDBC sets the autocommit to true, which means that every update is
    // committed immediately. In a transaction environment you probably would not want that
    // but for simplicity in this servlet I am working around autocommit value == true.
    try {
        if (isDEBUGON) trace("Trying the JDBC driver.");
        Class.forName(driver);

        //  The following line enables driver logging for debugging.
        //  It is not required, and may be commented out if desired.
        //  Note: this WILL impact performance! Use when required.
        //  if (isDEBUGON) trace("Setting the Log Stream.");
        //  DriverManager.setLogStream(System.out);

        if (isDEBUGON) trace("Getting a database connection.");
        connection = DriverManager.getConnection(url, properties);
        if (isDEBUGON) trace("Checking for connection warnings.");
        checkForWarning(connection.getWarnings());
        if (isDEBUGON) trace("Connected to database.");
        // Safe at this point to set isloggedOn flag to true indicating that we no
        // longer need to request the user name, password, driver or url.
        isLoggedOn = true;
    }
    catch (SQLException anSQLException) {
        if (isDEBUGON) trace("An SQL Exception was thrown:\n" + anSQLException);
        processSQLException(anSQLException);
    }
    catch (Exception anException) {
        // some other kind of exception occurred so dump the call stack
        anException.printStackTrace();
    }
}
/**
 * This method was created by a SmartGuide.
 * Destroys the servlet upon server shutdown, frees resources like database connections.
 */
public void destroy() {

    if (isDEBUGON) trace("Destroying the servlet.");
    if (isDEBUGON) trace("Logging user off and closing database connections.");

    try {
       // Close the Statement and make object eligible for garbage collection
        if (statement != null) {
            statement.close();
            statement = null;
        }

        // Close the ResultSet and make object eligible for garbage collection.
        if (resultSet != null) {
            resultSet.close();
            resultSet = null;
        }

        // Close the Connection and make object eligible for garbage collection.
        if (connection != null) {
            connection.close();
            connection = null;
        }

        // Try to close the ServletOutputStream.
        if (outstream != null) {
            try {
                outstream.close();
            }
            catch (IOException anIOException) {
                System.err.println ("\n*** IOException caught ***\n");
                System.err.println("IOException: " + anIOException.toString());
            }
        }

        // Set the user to a logged out status.
        isLoggedOn = false;

    }
    catch (SQLException anSQLException) {
        if (isDEBUGON) trace("An SQL Exception was thrown:\n" + anSQLException);
        processSQLException(anSQLException);
    }
}
/**
 * This method was created by a SmartGuide.
 * Executes a user-defined SQL query on a table.
 * @param anSQLQuery String
 */
private synchronized void executeTheQuery(String anSQLQuery) {

    try {
        if (isDEBUGON) trace("Executing a query.");
        statement = connection.createStatement();
        resultSet = statement.executeQuery(anSQLQuery);
        isSQLExceptionOccurred = false;
    }
    catch (SQLException anSQLException) {
        if (isDEBUGON) trace("An SQL Exception was thrown:\n" + anSQLException);
        processSQLException(anSQLException);
    }
}
/**
 * This method was created by a SmartGuide.
 * Executes a User-defined SQL update command on a table.
 * @param anUpdateCommand String
 */
private synchronized void executeTheUpdate(String anUpdateCommand) {

    try {
        if (isDEBUGON) trace("Executing an update, insert or delete command.");
        statement = connection.createStatement();
        int rowsAffected = statement.executeUpdate(anUpdateCommand);

        if (isDEBUGON) trace("Number of Rows returned: " + rowsAffected);

        // If no rows are returned it means the call to executeUpdate failed
        // for whatever reason, but did not actually do anything that would
        // cause an SQLException to be thrown.
        if (rowsAffected > 0)
            isExecuteUpdateSuccess = true;
        else
            isExecuteUpdateSuccess = false;
    }
    catch (SQLException anSQLException) {
        if (isDEBUGON) trace("An SQL Exception was thrown:\n" + anSQLException);
        isExecuteUpdateSuccess = false;
        processSQLException(anSQLException);
    }
}
/**
 * This method was created by a SmartGuide.
 * This method will gather the properties passed into the servlet as the request.
 */
private void getRequestParameters() {

    if (isDEBUGON) trace("Getting request parameters.");
    if (isDEBUGON) trace("Current Logon Status: " + isLoggedOn);

    String[] userName = request.getParameterValues("UID");
    String[] password = request.getParameterValues("password");

    // Do not overwrite key logon properties if already successfully logged on to the database
    // and the user has issued additional SQL. Additionally it serves to disconnect the user
    // and free up resources when the user indicates desire to be logged off which is accomplished
    // by passing an SQL value of null which occurs when the user calls the servlet directly as they
    // first did to initialize their use of it.
    if (!isLoggedOn) {
        if (userName != null)
            this.userName = userName[0];

        if (password != null)
            this.password = password[0];

        String[] driver = request.getParameterValues("driver");

        if (driver != null)
            this.driver = driver[0];

        String[] url = request.getParameterValues("URL");

        if (url != null)
            this.url = url[0];
    }

    // If SQL is null take that as a cue to log current user off and
    // send the initial logon form (again).
    String[] sql = request.getParameterValues("SQL");
    if (sql == null)  {
        this.userName = null;
        password = null;
        driver = null;
        url = null;

        isLoggedOn = false;

        if (isDEBUGON) trace("Logging user off and closing database connections.");

        // Since we are 'logging off', disconnect from the database, close the output stream
        // and free up resources by closing the Statement and ResultSet.
        try {
            if (statement != null) {
                statement.close();
                statement = null;
            }

            if (resultSet != null) {
                resultSet.close();
                resultSet = null;
            }

            if (connection != null) {
                connection.close();
                connection = null;
            }

            if (outstream != null) {
                try{
                    outstream.close();
                }
                catch(IOException anIOException) {
                    System.err.println ("\n*** IOException caught ***\n");
                    System.err.println("IOException: " + anIOException.toString());
                }
            }
        }
        catch (SQLException anSQLException) {
            if (isDEBUGON) if (isDEBUGON) trace("An SQL Exception was thrown:\n" + anSQLException);
            processSQLException(anSQLException);
        }
    }
    // else, get this sql and save it as the last sql
    else {
        this.sql = sql[0].trim();
        this.lastSQL = this.sql;
    }

    if (isDEBUGON) {
        trace("User Name: " + this.userName);
        trace("Password: " + this.password);
        trace("Driver: " + this.driver);
        trace("URL: " + this.url);
        trace("SQL: " + this.sql);
    }
}
/**
 * This method was created by a SmartGuide.
 * Returns information about this servlet.
 * @return String about the servlet.
 */
public String getServletInfo() {
    return "SQL Servlet version 1.0\n" +
           "(C) Sheldon Bradley Wosnick\n" +
           "IBM Toronto Lab\n" +
           "Toronto Ontario, Canada\n" +
           "February 1998";
}
/**
 * This method was created by a SmartGuide.
 * @param String sql
 * @return int
 */
private int getSQLOperation(String anSQLString) {

    int operation = NO_OPERATION;

    // Here we guarantee that the sqlOperation is at least 6 characters so that
    // the substring() method will not throw an exception by inserting some dummy chars.
    String sqlOperation = anSQLString + "XXXXXX";
    sqlOperation = sqlOperation.substring(0, 6);

    if ("SELECT".equalsIgnoreCase(sqlOperation))
        operation = SELECT;
    else if ("INSERT".equalsIgnoreCase(sqlOperation)  ||
             "UPDATE".equalsIgnoreCase(sqlOperation)  ||
             "DELETE".equalsIgnoreCase(sqlOperation))

             operation = INSERT_UPDATE_DELETE;

    return operation;
}
/**
 * This method was created by a SmartGuide.
 * Initialization of the servlet occurs here on an one-time basis.
 * @param aServletConfig ServletConfig
 */
public void init(ServletConfig aServletConfig) {

    if (isDEBUGON) trace("Calling init method now.");

    try{
        super.init(aServletConfig);
    }
    catch (ServletException aServletException) {
        System.err.println("ServletException: " + aServletException.getMessage());
    }
}
/**
 * This method was created by a SmartGuide.
 * Prepares http output to create new html page with results of query
 * after a user-entered SQL query or update command.
 * @exeception SQLException
 */
private void outResultSet() throws SQLException {

    try {
        if (isDEBUGON) trace("Getting an output stream for the HTML response page.");
        response.setContentType("text/html");
        outstream.println("<html>");
        outstream.println("<head><title>SQL Gateway Servlet version 1.0</title></head>");
        outstream.println("<body bgcolor=" + backgroundColor +">");
        outstream.println("<BASEFONT FACE=\"Verdana, Arial, Helvetica\">");
        outstream.println("<FONT FACE=\"Arial, Helvetica\" SIZE=3>");
        outstream.println("<h2 align=center><font color=#993366>SQL Query Results</font></h2>");
        outstream.println("<HR WIDTH=100%>");

        try {
            ResultSetMetaData resultSetMetaData = resultSet.getMetaData();
            int numberColumns = resultSetMetaData.getColumnCount();
            int columnNumber;

            if (isDEBUGON) {
                trace("Preparing some connection information.");
                connectionInfo();
            }

            outstream.println("<font color=#800040>");
            outstream.println("<BR>SQL PROCESSED: </font>" + sql);
            outstream.println("<BR><BR>");

            if (isDEBUGON) trace("Preparing an HTML table.");
            outstream.println("<TABLE>");
            outstream.println("<TABLE BORDER=1>");
            outstream.println("<tr bgcolor=" + tableHeadingBackgroundColor + ">");

            // create column names
            if (isDEBUGON) trace("Creating the column names");

            for (columnNumber = 1; columnNumber <= numberColumns; columnNumber++) {
                outstream.println("<TH><font color=" + tableHeadingForeroundColor + ">" + resultSetMetaData.getColumnLabel(columnNumber) + "</font>");
            }

            boolean more = resultSet.next();

            // fill cells in table with values extracted from result set
            if (isDEBUGON) trace("Filling in the table elements.");
            while (more) {
                outstream.println("<TR>");
                for (columnNumber = 1; columnNumber <= numberColumns; columnNumber++) {
                            outstream.println("<TD>" + resultSet.getString(columnNumber));
                }

                more = resultSet.next();
            }

            outstream.println("</TABLE>");
            outstream.println("<HR WIDTH=100%>");

            if (isDEBUGON) trace("Finished preparing the HTML table.");
        }
        catch (SQLException anSQLException) {
            if (isDEBUGON) trace("An SQL Exception was thrown:\n" + anSQLException);
            processSQLException(anSQLException);
        }

        outstream.println("</body></html>");
        if (isDEBUGON) trace("Finished with the output stream.");
        outstream.flush();
        outstream.close();
    }
    catch (IOException anIOException) {
        System.err.println ("\n*** IOException caught ***\n");
        System.err.println("IOException: " + anIOException.toString());
    }

    sendSQLForm();
}
/**
 * This method was created by a SmartGuide.
 * Provide the user with feedback that the operation was successful along with an SQL form.
 * @param success boolean
 */
private void outUpdateStatus(boolean isSuccess) {

    try {
        if (isDEBUGON) trace("Update, Insert or Delete succeeded: " + isSuccess);
        response.setContentType("text/html");
        outstream.println("<html>");
        outstream.println("<head><title>SQL Servlet version 1.0</title></head>");
        outstream.println("<body bgcolor=" + backgroundColor +">");
        outstream.println("<BASEFONT FACE=\"Verdana, Arial, Helvetica\">");
        outstream.println("<FONT FACE=\"Arial, Helvetica\" SIZE=3>");
        outstream.println("<h2 align=center><font color=#993366>Status of Update, Insert or Delete</font></h2>");

        outstream.println("<font color=#800040>");
        outstream.println("<BR>SQL PROCESSED: </font>" + sql);
        outstream.println("<BR><BR>");

        if (isSuccess)
            outstream.println("<p><h3 align=left>The Insert, Update or Delete operation SUCCEDED.</h3>");
        else
            outstream.println("<p><h3 align=left>The Insert, Update or Delete operation FAILED.</h3>");

        outstream.println("</body></html>");
        outstream.flush();
        outstream.close();
    }
    catch (IOException anIOException) {
        System.err.println ("\n*** IOException caught ***\n");
        System.err.println("IOException: " + anIOException.toString());
    }

    sendSQLForm();
}
/**
 * This method was created by a SmartGuide.
 * Process the SQL exception by extracting relevant information from it.
 * Display to the browser.
 * @param anSQLException SQLException
 */
private void processSQLException(SQLException anSQLException) {

    isSQLExceptionOccurred = true;

    try {
        if (isDEBUGON) trace("Processing an SQL Exception.");
        response.setContentType("text/html");
        outstream.println("<html>");
        outstream.println("<head><title>Servlet threw an SQLException</title></head>");
        outstream.println("<body bgcolor=" + backgroundColor +">");
        outstream.println("<BASEFONT FACE=\"Verdana, Arial, Helvetica\">");
        outstream.println("<FONT FACE=\"Arial, Helvetica\" SIZE=3>");
        outstream.println("<h2 align=center><font color=#993366>*** An SQL Exception Occurred *** </font></h2>");

        while (anSQLException != null) {
            outstream.println("<P>" + "SQLState: " +
                anSQLException.getSQLState() + "</P>");
            outstream.println("<P>Message:  " + anSQLException.getMessage() + "</P>");
            outstream.println("<P>Vendor:   " + anSQLException.getErrorCode() + "</P>");
            anSQLException = anSQLException.getNextException();
            outstream.flush();
            outstream.close();
        }

        outstream.println("</body></html>");
    }
    catch (IOException anIOException) {
        System.err.println ("\n*** IOException caught ***\n");
        System.err.println("IOException: " + anIOException.toString());
    }

    sendSQLForm();
}
/**
 * This method was created by a SmartGuide.
 * This method sends the first logon form to the user to enter various logon information to connect.
 */
private void sendHTMLLogonForm() {

    if (isDEBUGON) trace("Sending the initial logon and SQL form.");

    String serverName = request.getServerName();
    if (isDEBUGON) trace("Server Name: " + serverName);

    int serverPort = request.getServerPort();
    if (isDEBUGON) trace("Server Port: " + serverPort);

    try {
        response.setContentType("text/html");
        outstream.println("<html><head><title>" + logonFormTitle + "</title></head>");
        outstream.println("<body bgcolor=" + backgroundColor + " text=#666666><font face=verdana>");
        outstream.println("<h2 align=center><font color=#993366>" + logonFormHeading + "</font></h2>");
        outstream.println("<center>This SQL Gateway Servlet lets you view and manipulate a relational database");
        outstream.println("using SQL via JDBC with any datasource configured on the Web server (including an");
        outstream.println("ODBC datasource) by way of a JDBC driver such as those that ship");
        outstream.println("with DB2 Universal Database or using the JDBC-ODBC Bridge driver. This servlet");
        outstream.println("supports SQL queries, updates, deletes and inserts.</center>");
        outstream.println("<blockquote><p align=center><font color=#800040>Logon to a database:</font></p></blockquote>");
        outstream.println("<form action=" + "http://" +  serverName + ":" + serverPort + "/servlet/" + servletName + " method=POST>");
        outstream.println("<blockquote><div align=center><center><p>User ID: <input type=text name=UID size=10 tabindex=1> &nbsp;Password: <input type=password name=password size=10 tabindex=2></p>");
        outstream.println("</center></div></blockquote></blockquote>");
        outstream.println("<div align=center><center><p>URL: <input type=text name=URL size=15 tabindex=3> &nbsp;");
        outstream.println("</center></div></blockquote><div align=center><center><p>JDBC Driver: <select name=driver size=1 tabindex=5>");

        // first one will be the 'selected' one
        outstream.println("<option selected value=" + JDBCDrivers[0] + ">" + JDBCDrivers[0] + "</option>");
        for (int numberJDBCDrivers = 1; numberJDBCDrivers < JDBCDrivers.length; numberJDBCDrivers++)
            outstream.println("<option value=" + JDBCDrivers[numberJDBCDrivers] + ">" + JDBCDrivers[numberJDBCDrivers] + "</option>");

        outstream.println("</select>&nbsp;&nbsp;</center></div></blockquote>");
        outstream.println("<blockquote><p align=center><font color=#800040>Enter the SQL here:</font></p>");
        outstream.println("<div align=center><center><p><textarea rows=2 name=SQL cols=49 tabindex=9></textarea></p>");
        outstream.println("</center></div><blockquote>");
        outstream.println("<div align=center><center><p><input type=submit value=Submit SQL Query or Command tabindex=10> <input type=reset value=\"Clear Entries\" name=\"clearEntries\" tabindex=11></p>");
        outstream.println("</body></html>");
        outstream.flush();
        outstream.close();
    }
    catch (IOException anIOException) {
        System.err.println ("\n*** IOException caught ***\n");
        System.err.println("IOException: " + anIOException.toString());
    }
}
/**
 * This method was created by a SmartGuide.
 * This method will send an SQL editable area as HTML to the user to further submit SQL commands.
 */
private void sendSQLForm() {

    if (isDEBUGON) trace("Sending the SQL form.");

    String serverName = request.getServerName();
    if (isDEBUGON) trace("Server Name: " + serverName);

    int serverPort = request.getServerPort();
    if (isDEBUGON) trace("Server Port: " + serverPort);

    try {
        response.setContentType("text/html");
        outstream.println("<html><head><title>" + logonFormTitle + "</title></head>");
        outstream.println("<body bgcolor=" + backgroundColor + ">");
        outstream.println("<form action=" + "http://" +  serverName + ":" + serverPort + "/servlet/" + servletName + " method=POST>");
        outstream.println("<blockquote><p align=left><font color=#800040>Enter additional SQL here:</font></p>");
        outstream.println("<div align=left><left><p><textarea rows=2 name=SQL cols=49 tabindex=9>" + this.lastSQL + "</textarea></p>");
        outstream.println("</left></div><blockquote>");
        outstream.println("<div align=left><left><p><input type=submit value=Submit SQL Query or Command tabindex=10><input type=reset value=\"Previous SQL\" name=\"clearSQL\" tabindex=11></p>");
        outstream.println("</body><html>");
        outstream.flush();
        outstream.close();
    }
    catch (IOException anIOException) {
        System.err.println ("\n*** IOException caught ***\n");
        System.err.println("IOException: " + anIOException.toString());
    }
}
/**
 * This method was created by a SmartGuide.
 * Required method for a servlet, overridden to perform specialized JDBC work.
 * @param aRequest HttpServletRequest, aResponse HttpServletResponse
 * @exception ServletException, IOException
 */
public synchronized void service(HttpServletRequest aRequest, HttpServletResponse aResponse)
    throws ServletException, IOException {

    // Used for debugging. Turn off if desired.
    log("called");

    if (isDEBUGON) trace("Calling the service method.");

    // store class-level references to both the request and the response
    this.request = aRequest;
    this.response = aResponse;

    // always get a new output stream, because we always close once we've sent the page
    outstream = response.getOutputStream();

    getRequestParameters();
    // Send the full logon form if the user calls the servlet for
    // the first time.
    if ((!isLoggedOn) && (userName == null)) {
        sendHTMLLogonForm();
        return;
    }

    // if we reached this point at least the username is not null (could be
    // empty string, but not null)
    properties = new Properties();
    properties.put("UID", userName);
    // Do not force user to submit a password, especially if not required, in that
    // case just use empty string, else use the supplied password.
    if (password != null)
        properties.put("PWD", password);
    else
        properties.put("PWD", "");

    // Attempt a connection only if NOT already successfully connected.
    // This means that for each client, the database connection is left open until
    // no longer required, and this enhances performance and lowers overhead.
    if (connection == null)
        connectToDatabase();

    int sqlOperation = getSQLOperation(sql);

  if (sqlOperation == SELECT) {
        executeTheQuery(sql);
        if (!isSQLExceptionOccurred) {
            try {
                 outResultSet();
            }
            catch (SQLException anSQLException) {
                if (isDEBUGON) trace("An SQL Exception was thrown.");
                processSQLException(anSQLException);
            }
        }
    }
    else if (sqlOperation == INSERT_UPDATE_DELETE) {
        executeTheUpdate(sql);
        if (isExecuteUpdateSuccess)
            outUpdateStatus(true);
        else
            outUpdateStatus(false);
    }
    else if (sqlOperation == NO_OPERATION) {
        System.err.println("Syntax Error:  Must be an UPDATE, INSERT, DELETE or SELECT");
        SQLException anSQLException = new SQLException("Possible syntax error occured: Enter 'SELECT', 'INSERT', 'UPDATE' or 'DELETE'");
        processSQLException(anSQLException);
    }

    // Close ResultSet and Statement and make them available for garbage collection.
    try {
        if (resultSet != null) {
            resultSet.close();
            resultSet = null;
        }

        if (statement != null) {
            statement.close();
            statement = null;
        }
    }
    catch (SQLException anSQLException) {
        if (isDEBUGON) trace("An SQL Exception was thrown:\n" + anSQLException);
        processSQLException(anSQLException);
    }
}
/**
 * This method was created by a SmartGuide.
 * This method is used to output trace messages to standard output.
 * @param aTraceString String
 */
public void trace(String aTraceString) {
    System.out.println("\nDEBUG TRACE: \n " + aTraceString);
}
}